/*
        Pin, register and other definitions for ESP8266EX chip on In-Circuit radino WiFi modules
        for more information: www.in-circuit.de or www.radino.cc

	Copyright (c) 2014 In-Circuit GmbH

	Permission is hereby granted, free of charge, to any person obtaining a copy of this software
	and associated documentation files (the "Software"), to deal in the Software without restriction,
	including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
	and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
	LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
	IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
	WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

#ifndef IC_ESP_h
#define IC_ESP_h

#include "Arduino.h"
#include <SPI_UART.h>

//enum Responsetypes{
	//READRESPONSE,
	//WRITERESPONSE
	//
//};

// DEBUG DEFINES

// DEBUG DEFINES END

// Pin definitions of ESP8266EX
//       Pin-Name   Arduino-Pin-Number
#define  CH_PD      A4
#define  CH_RST     4
#define  GPIO0      9
//       Pin-Name   SC16IS750-GPIO-Number
#define  GPIO2      0

enum ESP_modes{
	ESP_RAW_MODE = 0,
	ESP_IC_PROTOCOL_MODE,
	ESP_FIMWAREUPDATE_MODE
	};

enum ESP_parser_states{
	IC_PARSER_IDLE = 0, // Idle, waiting for new command
	IC_PARSER_ICSTART, // 'I'
	
	// Got Event
	IC_PARSER_PARA_1, // register number / event number
	IC_PARSER_PARA_2, // DataInfo High Byte / Error-Code High Byte
	IC_PARSER_PARA_3, // DataInfo Low Byte / Error-Code Low Byte
	
	IC_PARSER_COPYTOWORKER,

	IC_PARSER_CHECKANDLEAVE, // Check Checksum & done
	IC_PARSER_CHECKANDCOPY, // Check checksum & update data registers
	IC_PARSER_DATA, // Data
	IC_PARSER_DATA_STRING_READY, // Datatype: String - data ready
//	IC_PARSER_STRINGBURST,
//	IC_PARSER_EXEC
	IC_PARSER_INACTIVE = 0x80,
	IC_PARSER_DISABLE_PARSER,
	};

enum ESP_wifi_modes{
	ESP_WIFI_STATION = 1,
  ESP_WIFI_AP = 2,
	ESP_WIFI_DUAL = 3,
};

enum ESP_read_modes{
	READ_BLOCKING,
	READ_ASYNC,	
	};
	
enum ESP_write_modes{
	WRITE_BLOCKING,
	WRITE_ASYNC,
	WRITE_IGNORE_RESPONSE,
};

enum ESP_IC_registers{
	DUMMY_STROBE_AP =   0x00,
	
	WIFI_MODE =			0x01,
	WIFI_SSID_AP =		0x02,
	WIFI_PASSWD_AP =	0x03,
	WIFI_SSID_JOIN =	0x06,
	WIFI_PASSWD_JOIN =	0x07,

	IP_ADDR_READ =		0x10,
	
	TCP_PUSH =		0x4B,
	HTTP_STROBE = 		0x61,
	HTTP_SERVERPORT =	0x62,
	HTTP_UPLOAD =		0x63,

	SYSTEM_STROBE =		0x72,
	FIRMWARE_VERSION =	0x7F,
	};

enum ESP_event_types{
	EVENT_NONE = 0,
	EVENT_TCP_NEWDATA = 1,
	EVENT_UDP_NEWDATA = 2,
	EVENT_HTTP_SET_REQUEST = 3,
	EVENT_HTTP_SET_RESPONSE = 4,
	EVENT_HTTP_GET_REQUEST = 5,
	EVENT_HTTP_GET_RESPONSE = 6,	
	};
	
enum ESP_event_modes{
	EVENT_MODE_IGNORE = 0,
	EVENT_MODE_HANDLE_ASYNC,	
	};

struct event {
	// IE event type
	ESP_event_types type;
	// IE GET / SET register
	unsigned reg;
	unsigned value;
};

struct event_state {
	/* interface */
	unsigned data_available;
	// set to 1 by newEventAvailable, reset by sendEventResponse
	// if newEventAvailable if called again and value is still 1
	// we know that this is a request the user sketch won't handle
	unsigned data_read;
	// to indicate the parser that data can be overwritten
	unsigned data_handled;

	struct event event;
	/* data buffer for IE requests */
	unsigned char buffer[64];
	unsigned buffer_index;
	unsigned buffer_len;
};

struct command_state {
	// TODO store header etc.
	/* interface */
	unsigned data_available;
	unsigned data_handled;

	//cmd;
	// small buffer for read/write requests
	unsigned long buffer_small;
};

// General macros
#define IP(a,b,c,d)	(((unsigned long)a<<24)|((unsigned long)b<<16)|((unsigned long)c<<8)|((unsigned long)d<<0)) // Build 32Bit IP value from seperate ip segments

// Parser help-defines
#define	IC_PARSER_SHIFT_DATATYPE	13	// Shift DataInfo for DataType
#define IC_PARSER_MASK_DATALEFT		0x1FFF // Cut top 3 bits
#define IC_PARSER_MASK_BAD_STATE    0x80 // Mask for states where normal operation is not possible

// DataTypes
#define IC_DT_STRING	  0x00
#define IC_DT_UINT32      0x01
#define IC_DT_SINT32      0x02
#define IC_DT_FLOAT       0x03
#define IC_DT_STRINGBURST 0x04

// Handshake-Line
// The host who pulls line LOW is currently Master of communication
// until he lets go again
#define HARDWARE_HANDSHAKE 9	// Arduino Pin 9 / ESP8266EX GPIO0

class IC_ESP
{	
	public:
		static IC_ESP* callback_instance; // Variable to remember own instance to refer to callback-functions
		
		IC_ESP(); // Constructor
		
		unsigned char init(); // Init ESP8266EX
		
		unsigned char ready(); // Check if ESP is ready for communication; 0: not ready - 1: ready
		
		unsigned char setMode(unsigned char mode); // Set mode to normal / firmwareUpdate
		
		static unsigned char handleInterrupt(unsigned char val); // function which handles SC16 interrupts
		
		unsigned char parser(void); // main parser
		void parser_addCheck(unsigned char newChar); // Add next char to checksum
		unsigned char parser_checkChecksum(unsigned char checksum); // returns True if "checksum" equals last calculated checksum

		unsigned char writeRegister(unsigned char reg, String data, unsigned char writeMode = WRITE_BLOCKING, unsigned short timeoutms = 500); // Read Register of ESP8266EX; Returns success or Error Code	
		unsigned char writeRegister(unsigned char reg, unsigned long data, unsigned char writeMode = WRITE_BLOCKING, unsigned short timeoutms = 500);
		
		unsigned char readRegister(unsigned char reg, unsigned char readMode = READ_BLOCKING, unsigned short timeoutms = 500); // returns status -> "Data now ready" or "got ERROR"; data itself must be read with "readNewData()"
		unsigned long readNewData(); // Read 4 dataBytes after ReadRegister-request
		
		void SPI_setBaudrate(unsigned long baudrate); // Set baudrate of SC16IS750
		
		void write(unsigned char character); // Directly write to ESP8266EX UART
		unsigned char read(); // Directly read from ESP8266EX UART
		unsigned char available(); // Data available on ESP8266EX UART?
				
		// Application Layer Functions
		unsigned char AP_setSSID(String ssid); // Set SSID for AP-Mode -  max. 32 chars
		unsigned char AP_setPW(String pw); // Set PW for AP-mode - max. 64 chars
		unsigned char ST_setSSID(String ssid); // Set SSID for JOIN-Mode / station -  max. 32 chars		
		unsigned char ST_setPW(String pw); // Set PW for JOIN-mode / station - max. 64 chars
		unsigned char NET_mode(ESP_wifi_modes mode); // select network mode Station/AP
		unsigned char NET_start(); // start / restart network and load new settings

		unsigned char NET_wifiStart();
		
		void TCP_push(unsigned long ip, unsigned short port, String data); // Send "data" to target IP & port via TCP and don't wait for response
		
		unsigned char HTTPD_setServerPort(unsigned short port); // set HTTP server port, default 80
		unsigned char HTTPD_startServer(); // start HTTP server
		unsigned char HTTPD_updateWebsite(char *data);
		unsigned char HTTPD_updateWebsite(String *data); // Upload new website to internal webserver

		// Debugging only !!!
		unsigned char debugInit(unsigned char mode);
		unsigned char debugHandle(unsigned char);
		
		// \/------------ not implemented yet ---------------\/
		
		void setBaudRate(unsigned long baudrate); // Set Baudrate of ESP8266 via command

		unsigned char writeRegister(unsigned char reg, signed long data, unsigned char writeMode = WRITE_BLOCKING, unsigned short timeoutms = 500);
		unsigned char writeRegister(unsigned char reg, float data, unsigned char writeMode = WRITE_BLOCKING, unsigned short timeoutms = 500);
		
		unsigned short getWriteResponse(); // Return 2 Byte Write-Response
		
		unsigned char newDataAvailable();
		unsigned char getDataType();	// Get dataType of currently received data
		unsigned char writeToSocket(unsigned char socketNumber, String data); // Write data to specified TCP socket
		void factoryReset(); // Tell ESP8266 to load factory settings

		// Application Layer Functions
		unsigned char JOIN_setSSID(String ssid); // Set SSID for JOIN-Mode

		// check if event available
		unsigned newEventAvailable();
		// return event type; only valid if new event is available
		struct event getEvent();
		// send response; only valid if new event is available
		void sendEventResponse(unsigned long val);
		void sendEventResponse(String str);

		// debug
		void showIPAddress();

		void disableInterrupt();
		void enableInterrupt();

		unsigned getFirmwareVersion();
		unsigned long getIPAddress();
		void getIPAddress(unsigned char *buf, unsigned long ip);

	private:
		unsigned char handshakeAlreadyActive(); // Read Handshake-Line and return status (1 == HIGH == used; 0 == LOW == FREE)
		void setHandshake(); // Set Handshake-line HIGH
		void releaseHandshake(); // Let go of Handshake-line
		
		void initVariables(); // Init all variables
		
		void clearParserReceiveRegs(); // Reset Receive buffers; set "_worker_data_valid" to 0
		
		SPI_UART _espUART; // SPI-UART for ESP8266EX in radino Wifi
		
		// Settings
		unsigned long _baudrate; // Baudrate for SPI-UART Bridge / current Baudrate with ESP8266
		unsigned char _mode; // ESP current mode
		unsigned char _event_mode; // Event Mode
		
		// Flags
//		volatile unsigned char _got_Response;
//		volatile unsigned char _waiting_for_response; // TRUE while waiting for response, false if not waiting for response
		volatile unsigned char _event_handle_required; // set to Event Type if event was received and needs to be read -> 
		
		// States -> Parser
		volatile unsigned char  _parser_state; // State for parser
		unsigned char  _parser_cmd_type; // command type
		unsigned char  _parser_reg; // register number / IE: event type
		unsigned short _parser_cmd_dataHeader; // 2 Bytes 'dataInfo' / Error Code
		unsigned char  _parser_cmd_dataType; // DataType
		unsigned short _parser_cmd_dataLeft; // Data amount
		volatile unsigned long  _parser_data_buffer; // 32 Bit data of non-String-Commands
		unsigned char  _parser_checksum; // Checksum
		
		//unsigned char _parser_ie_http_socketid; // buffer for socketID
		//unsigned long _parser_ie_http_reg; // register number -  16 bit
		unsigned long _parser_timeout;
		
		// Flags for worker
		// Parser READ-only; Worker WRITE-only:
		volatile unsigned char _worker_handling_active; // True while worker is accessing _worker_* registers
		// Parser WRITE-only; Worker READ-only:
		volatile unsigned char _worker_data_valid; // True after Parser has written new data to the registers
		
		// States / Data for worker
		volatile unsigned char  _worker_cmd_type; // command type
		volatile unsigned char  _worker_reg; // register number
		unsigned short _worker_cmd_dataHeader; // 2 Bytes 'dataInfo' / Error Code
		unsigned char  _worker_cmd_dataType; // DataType
		volatile unsigned short _worker_cmd_dataLeft; // Data amount
		unsigned long  _worker_data_buffer; // 32 Bit data of non-String-Commands

		// worker state
		struct command_state _cs;
		struct event_state _es;
		// reset state
		void initStates();
		void initEventState();
		void initCommandState();

		void IE_GET_Response(unsigned char *socket, unsigned char *reg, unsigned char *val, unsigned char *uip);
		
		//unsigned char _worker_ie_http_socketid; // buffer for socketID
		//unsigned short _worker_ie_http_reg; // register number -  16 bit
				
		// Data
//		volatile unsigned char _current_dataByte_count; // Variable which holds current count of remaining data bytes
		
		// HTTP
//		String _current_Register; // Current register name that is handled
		
		
	};
		
#endif
